from datetime import datetime
from sqlalchemy import desc, ForeignKeyConstraint
from config import db
from flask_login import UserMixin
from werkzeug.security import check_password_hash, generate_password_hash
import logging
logger = logging.getLogger("__models__")


class GroupUser(db.Model):
    __tablename__ = "GroupUser"
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(100), nullable=False)
    active = db.Column(db.String(10), nullable=False)
    last_Updated_date = db.Column(db.DateTime, default=datetime.now, nullable=False)
    GroupUsers = db.relationship('User', backref='GroupUser', lazy='dynamic')

    @staticmethod
    def get_by_id(user_id):
        return GroupUser.query.filter_by(id=user_id).first()

    @staticmethod
    def get_by_id_active(user_id, active):
        return GroupUser.query.filter_by(id=user_id, active=active).first()

    @staticmethod
    def get_by_name(name):
        return GroupUser.query.filter_by(name=name).first()

    @staticmethod
    def get_by_name_active(name, active):
        return GroupUser.query.filter_by(name=name, active=active).first()

    def __init__(self, name, active):
        self.name = name
        self.active = active

    def __repr__(self):
        return "<GroupUser '{}' : '{}'>".format(self.id, self.name, self.active)


class User(db.Model, UserMixin):
    __tablename__ = "User"
    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(120), unique=True)
    email = db.Column(db.String(120), unique=True)
    password_hash = db.Column(db.String, nullable=False)
    firstname = db.Column(db.String(120), nullable=False)
    lastname = db.Column(db.String(120), nullable=False)
    user_group_id = db.Column(db.Integer, db.ForeignKey('GroupUser.id'), nullable=False)
    last_Updated = db.Column(db.DateTime, default=datetime.now, nullable=False)
    last_Updated_by_user = db.Column(db.String(120), nullable=False, default="wohler.admin")
    networkSetups = db.relationship('NetworkSetUp', backref='user', lazy='dynamic')
    products = db.relationship('Product', backref='user', lazy='dynamic')
    specifications = db.relationship('Specification', backref='user', lazy='dynamic')
    # screens = db.relationship('Screen', backref='user', lazy='dynamic')
    channels = db.relationship('Channel', backref='user', lazy='dynamic')
    configs = db.relationship('Configuration', backref='user', lazy='dynamic')
    config_channels = db.relationship('ConfigChannel', backref='user', lazy='dynamic')

    @property
    def password(self):
        raise AttributeError('password: write-only field')

    @password.setter
    def password(self, password):
        self.password_hash = generate_password_hash(password)

    def check_password(self, password):
        return check_password_hash(self.password_hash, password)

    @staticmethod
    def get_by_username(username):
        return User.query.filter_by(username=username).first()

    @staticmethod
    def get_by_email(email):
        return User.query.filter_by(email=email).first()

    @staticmethod
    def get_by_username_group(usr_name, usr_group_id):
        return User.query.filter_by(email=usr_name, user_group_id=usr_group_id).first()

    def __repr__(self):
        return '<User %r>' % self.username

    def __init__(self, email, password, firstname, lastname, username="administrator", user_group_id=3):
        self.username = username
        self.email = email
        self.password_hash = generate_password_hash(password)
        self.firstname = firstname
        self.lastname = lastname
        self.user_group_id = user_group_id

    def initials(self):
        return "{}. {}.".format(self.firstname[0], self.lastname[0])


class NetworkSetUp(db.Model):
    __tablename__ = "NetworkSetup"
    id = db.Column(db.Integer, primary_key=True)
    network_type = db.Column(db.String(50), nullable=False)
    network_ip_address = db.Column(db.String(20), nullable=False)
    mask_ip_address = db.Column(db.String(20), nullable=False)
    gateway_ip_address = db.Column(db.String(20), nullable=False)
    multicast_ip_address = db.Column(db.String(20), nullable=False)
    port_number = db.Column(db.Integer, nullable=True)
    last_Updated_date = db.Column(db.DateTime, default=datetime.now, nullable=False)
    last_Updated_by_user = db.Column(db.Integer, db.ForeignKey('User.id'), nullable=False)

    @staticmethod
    def newest(num):
        return NetworkSetUp.query.order_by(desc(NetworkSetUp.last_Updated)).limit(num)

    @staticmethod
    def get_all(num):
        return NetworkSetUp.query.order_by(NetworkSetUp.id).limit(num)

    @staticmethod
    def get_by_netid(net_id):
        return NetworkSetUp.query.filter_by(id=net_id).first()

    @staticmethod
    def get_by_type(net_type):
        return NetworkSetUp.query.filter_by(network_type=net_type).first()

    def __init__(self, network_type, network_ip_address, mask_ip_address, gateway_ip_address, multicast_ip_address,
                 user_id, port_number=None):
        self.network_type = network_type
        self.network_ip_address = network_ip_address
        self.mask_ip_address = mask_ip_address
        self.gateway_ip_address = gateway_ip_address
        self.multicast_ip_address = multicast_ip_address
        self.port_number = port_number
        self.last_Updated_by_user = user_id

    def __repr__(self):
        return "<NetworkSetUp '{}' : '{}'>".format(self.network_type, self.network_ip_address, self.mask_ip_address,
                                                   self.gateway_ip_address, self.multicast_ip_address, self.port_number)


class Product(db.Model):
    __tablename__ = "Product"
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(100), nullable=False)
    model = db.Column(db.String(100), nullable=False)
    serial_number = db.Column(db.String(50), nullable=True)
    hardware_version = db.Column(db.String(20), nullable=True)
    system_version = db.Column(db.String(20), nullable=True)
    channel_number = db.Column(db.Integer, nullable=True)
    screen_number = db.Column(db.Integer, nullable=True)
    passcode = db.Column(db.Integer, nullable=False, default=1234)
    active = db.Column(db.String(1), nullable=False, default='N')
    last_Updated_date = db.Column(db.DateTime, default=datetime.now, nullable=False)
    last_Updated_by_user = db.Column(db.Integer, db.ForeignKey('User.id'), nullable=False)
    product_features = db.relationship('ProductFeature', backref='Product', lazy='dynamic')

    @staticmethod
    def get_active():
        return Product.query.filter_by(active='Y').first()

    @staticmethod
    def get_by_id(product_id):
        return Product.query.filter_by(id=product_id).first()

    @staticmethod
    def get_by_name(name):
        return Product.query.filter_by(name=name).first()

    @staticmethod
    def get_by_name_model(name, model):
        return Product.query.filter_by(name=name, model=model).first()

    @staticmethod
    def get_all():
        return Product.query.order_by(Product.id)

    def __repr__(self):
        return "<Product '{}' : '{}'>".format(self.id, self.name, self.model, self.serial_number,
                                              self.hardware_version, self.system_version, self.channel_number,
                                              self.screen_number, self.passcode)


class Feature(db.Model):
    __tablename__ = "Feature"
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(50), nullable=False)
    eeprom = db.Column(db.String(50), default="0", nullable=False)
    last_Updated_date = db.Column(db.DateTime, default=datetime.now, nullable=False)
    last_Updated_by_user = db.Column(db.Integer, db.ForeignKey('User.id'), nullable=False)
    feature_products = db.relationship('ProductFeature', backref='Feature', lazy='dynamic')

    @staticmethod
    def get_all():
        return Feature.query.order_by(Feature.id)

    @staticmethod
    def get_by_id(feature_id):
        return Feature.query.filter_by(id=feature_id).first()

    @staticmethod
    def get_by_name(name):
        return Feature.query.filter_by(name=name).first()


class ProductFeature(db.Model):
    __tablename__ = "ProductFeature"
    product_id = db.Column(db.Integer, db.ForeignKey('Product.id'), primary_key=True, autoincrement=False)
    feature_id = db.Column(db.Integer, db.ForeignKey('Feature.id'), primary_key=True, autoincrement=False)
    license_key = db.Column(db.String(100), nullable=False)
    last_Updated_date = db.Column(db.DateTime, default=datetime.now, nullable=False)
    last_Updated_by_user = db.Column(db.Integer, db.ForeignKey('User.id'), nullable=False)

    @staticmethod
    def get_by_product(product_id):
        return ProductFeature.query.filter_by(product_id=product_id).all()

    @staticmethod
    def get_by_product_feature(product_id, feature_id):
        return ProductFeature.query.filter_by(product_id=product_id, feature_id=feature_id).first()

    @staticmethod
    def get_all():
        return ProductFeature.query.order_by(ProductFeature.product_id)

    def __init__(self, product_id, feature_id, license_key, user_id):
        self.product_id = product_id
        self.feature_id = feature_id
        self.license_key = license_key
        self.last_Updated_by_user = user_id


class Specification(db.Model):
    __tablename__ = "Specification"
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(100), nullable=False)
    type = db.Column(db.String(50), nullable=False)
    last_Updated_date = db.Column(db.DateTime, default=datetime.now, nullable=False)
    last_Updated_by_user = db.Column(db.Integer, db.ForeignKey('User.id'), nullable=False)

    # spec_details = db.relationship('SpecificationDetail', backref='specification', lazy='dynamic')

    @staticmethod
    def get_by_id(spec_id):
        return Specification.query.filter_by(id=spec_id).first()

    @staticmethod
    def get_by_name(name):
        return Specification.query.filter_by(name=name).all()

    @staticmethod
    def get_by_type(type):
        return Specification.query.filter_by(type=type).all()

    @staticmethod
    def get_by_type_name(type, name):
        return Specification.query.filter_by(type=type, name=name).first()

    def __repr__(self):
        return "<Specification '{}' : '{}'>".format(self.id, self.name, self.type)


class Channel(db.Model):
    __tablename__ = "Channel"
    id = db.Column(db.Integer, primary_key=True)
    input_type_id = db.Column(db.Integer, db.ForeignKey('Specification.id'), primary_key=True, autoincrement=False)
    name = db.Column(db.String(100), nullable=True, default="MADI")
    name2 = db.Column(db.String(100), nullable=True)
    unit_id = db.Column(db.Integer, db.ForeignKey('Unit.id'), nullable=False)
    last_Updated_date = db.Column(db.DateTime, default=datetime.now, nullable=False)
    last_Updated_by_user = db.Column(db.Integer, db.ForeignKey('User.id'), default=1)
    config_channels = db.relationship('ConfigChannel', backref='Channel', lazy='dynamic')

    @staticmethod
    def get_by_id(id):
        return Channel.query.filter_by(id=id).first()

    @staticmethod
    def get_by_id_input_type_id(id, input_type_id):
        return Channel.query.filter_by(id=id, input_type_id=input_type_id).first()

    @staticmethod
    def get_by_name(name, channel_id, input_type_id):
        return Channel.query.filter_by(name=name, id=channel_id, input_type_id=input_type_id).first()

    @staticmethod
    def get_by_name2(name2, channel_id, input_type_id):
        return Channel.query.filter_by(name2=name2, id=channel_id, input_type_id=input_type_id).first()

    @staticmethod
    def get_by_input_type(input_type_id):
        return Channel.query.filter_by(input_type_id=input_type_id)

    @staticmethod
    def get_all():
        return Channel.query.order_by(Channel.id, Channel.input_type_id)

    def __repr__(self):
        return "<Channel '{}' : '{}'>".format(self.id, self.input_type_id, self.name, self.name2, self.unit_id)


class Configuration(db.Model):
    __tablename__ = "Configuration"
    id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    name = db.Column(db.String(100), nullable=False)
    active = db.Column(db.String(1), nullable=False, default="N")
    level_indicator = db.Column(db.String(1), nullable=False, default='Y')
    mix_indicator = db.Column(db.String(1), nullable=False, default='Y')
    sdi_indicator = db.Column(db.String(1), nullable=False, default=1)
    dolby_input_type = db.Column(db.String(20), nullable=True)
    last_Updated_date = db.Column(db.DateTime, default=datetime.now, nullable=False)
    last_Updated_by_user = db.Column(db.Integer, db.ForeignKey('User.id'), default=1)
    config_channels = db.relationship('ConfigChannel', backref='Configuration', lazy='dynamic')

    @staticmethod
    def get_by_id(id):
        return Configuration.query.filter_by(id=id).first()

    @staticmethod
    def count_star():
        return Configuration.query.count()

    @staticmethod
    def get_all():
        return Configuration.query.order_by(Configuration.id)

    @staticmethod
    def get_active():
        return Configuration.query.filter_by(active='Y').first()

    def __repr__(self):
        return "<Configuration '{}' : '{}'>".format(self.id, self.name, self.active)


class ConfigChannel(db.Model):
    __tablename__ = "ConfigChannel"
    config_id = db.Column(db.Integer, db.ForeignKey('Configuration.id'), primary_key=True, autoincrement=False)
    screen_id = db.Column(db.Integer, nullable=True, primary_key=True, autoincrement=False)
    channel_id = db.Column(db.Integer)
    display_name = db.Column(db.String(100), nullable=False)
    label_1 = db.Column(db.String(100), nullable=False)
    label_2 = db.Column(db.String(100), nullable=False)
    channel_2 = db.Column(db.Integer, nullable=True)
    mix_id = db.Column(db.Integer, db.ForeignKey('Specification.id'))
    mix_type_id = db.Column(db.Integer, db.ForeignKey('Specification.id'))
    audio_id = db.Column(db.Integer, db.ForeignKey('Specification.id'))
    input_type_id = db.Column(db.Integer)
    audio_input_type_id = db.Column(db.Integer, nullable=True)
    volume = db.Column(db.Integer, nullable=True)
    clock_reference = db.Column(db.String(1), nullable=True)
    cluster_id = db.Column(db.Integer, nullable=True)
    cluster_name = db.Column(db.String(20), nullable=True)
    cluster_type = db.Column(db.Integer, db.ForeignKey('Specification.id'), nullable=True)
    cluster_label = db.Column(db.String(50), nullable=True)
    last_Updated_date = db.Column(db.DateTime, default=datetime.now, nullable=False)
    last_Updated_by_user = db.Column(db.Integer, db.ForeignKey('User.id'), default=1)
    __table_args__ = (ForeignKeyConstraint([channel_id, input_type_id], [Channel.id, Channel.input_type_id]), {})

    @staticmethod
    def get_by_config(config):
        return ConfigChannel.query.filter_by(config_id=config).all()

    @staticmethod
    def get_by_config_screen(config, screen):
        return ConfigChannel.query.filter_by(config_id=config, screen_id=screen).first()

    @staticmethod
    def get_by_type_config(type, config):
        return ConfigChannel.query.filter_by(input_type_id=type, config_id=config).all()

    @staticmethod
    def get_by_input_type_channel(input_type, channel):
        return ConfigChannel.query.filter_by(input_type_id=input_type, channel_id=channel).first()

    @staticmethod
    def get_all_by_input_type(input_type):
        return ConfigChannel.query.filter_by(input_type_id=input_type).all()

    @staticmethod
    def get_all():
        return ConfigChannel.query.order_by(ConfigChannel.config_id, ConfigChannel.input_type_id,
                                            ConfigChannel.channel_id)

    def __init__(self, config_id, channel_id, display_name, label_1, label_2, screen_id, mix_id, mix_type_id,
                 input_type_id, audio_input_type_id, channel_2=None, audio_id=None, volume=None, clock_reference=None,
                 cluster_id=None, cluster_name=None, cluster_type=None, cluster_label=None):
        self.config_id = config_id
        self.channel_id = channel_id
        self.display_name = display_name
        self.label_1 = label_1
        self.label_2 = label_2
        self.channel_2 = channel_2
        self.screen_id = screen_id
        self.mix_id = mix_id
        self.mix_type_id = mix_type_id
        self.audio_id = audio_id
        self.input_type_id = input_type_id
        self.audio_input_type_id = audio_input_type_id
        self.volume = volume
        self.clock_reference = clock_reference
        self.cluster_id = cluster_id
        self.cluster_name = cluster_name
        self.cluster_type = cluster_type
        self.cluster_label = cluster_label


# From presetmodels.py

class Preset(db.Model):
    __bind_key__ = 'presets_1_0'
    Id = db.Column(db.Integer, primary_key=True)
    ProgramId = db.Column(db.Integer, db.ForeignKey('program.Id'))
    FactoryDefault = db.Column(db.Integer, default=0, nullable=False)
    Name = db.Column(db.String(64), nullable=False)
    Data = db.Column(db.BLOB, nullable=True)
    last_Updated_date = db.Column(db.DateTime, default=datetime.now, nullable=False)
    __table_args__ = (db.UniqueConstraint('ProgramId', 'Name', name='_preset_uc'),
                 )

    def __init__(self, Id, Name, Data, ProgramId=None):
        self.Id = Id
        self.Name = Name
        self.Data = Data
        self.ProgramId = ProgramId

    def __repr__(self):
        return '<Id {}, Name {}, ProgramId {}>'.format(self.Id, self.Name, self.ProgramId)

    @staticmethod
    def get_by_id(config_id):
        try:
            return Preset.query.filter_by(Id=config_id).first()
        except Exception as e :
            logger.exception("Preset/get_by_id : %s ", str(e))
            return None

    @staticmethod
    def get_by_name(config_name):
        return Preset.query.filter_by(Name=config_name).all()

    @staticmethod
    def count():
        return Preset.query.count()

    @staticmethod
    def get_all():
        return Preset.query.order_by(Preset.Id)

    @staticmethod
    def delete_by_id(db, config_id):
        try:
            prst = Preset.query.filter_by(Id=config_id).first()
            if prst != None:
                db.session.delete(prst)
                db.session.commit()
                return True
            else:
                return False
        except Exception as e :
            db.session.rollback()
            logger.exception("Preset/delete_by_id : %s ", str(e))
            return False

    @staticmethod
    def rename_by_id(db, config_id, newname):
        try:
            prst = db.session.query(Preset).filter_by(Id=config_id).first()
            if prst != None:
                prst.Name = newname
                db.session.commit()
                return True
            else:
                return False
        except Exception as e :
            db.session.rollback()
            logger.exception("Preset/delete_by_id : %s ", str(e))
            return False


