var Util = function () {
    this.logger = Logger.get('Util');
};

/**
 * Return true if address is in my network.
 * @param myAddress My IP address
 * @param otherAddress The other IP address to check
 * @param prefixLength The length of the IP prefix in bits. (24, 16, etc.)
 * @returns true | false
 */
Util.prototype.isAddressInNetwork = function (myAddress, otherAddress, prefixLength) {
    try {
        var my = ipaddr.parse(myAddress),
            other = ipaddr.parse(otherAddress);
        return my.match(other, prefixLength);
    } catch (ex) {
        this.logger.error('isAddressInNetwork failed', ex);
        return false;
    }
};

/**
 *
 * @param config The configuration object
 * {
 *      cb: {
 *         // Mandatory fields
 *         status: "status-code",
 *         target: "ip-or-hostname",
 *         file: "base-filename"
 *         // ... and anything else you want to add to a particular
 *         // message's payload
 *      }
 * }
 * @constructor
 */
var FwUpdater = function (config) {
    config = config || {
            cb: {
                status: function (p) {
                    console.log(p.target + ': status=' + p.status, p);
                },
                //uploadStatus: function (target, file, status, data) {
                //    console.log('uploadStatus', target, status, data);
                //},
                //applyStatus: function (target, file, status, data) {
                //    console.log('applyStatus', target, status, data);
                //},
            }
        };
    this.config = config;
    this.logger = Logger.get('FwUpdater');
};

/**
 * pushFileToTarget
 *   - upload (push) a base filename from current unit's USB drive
 *     to the target unit.
 * @param target Hostname/ip address of the target box
 * @param file The base filename to push
 * @returns {Promise.<T>|*}
 */
FwUpdater.prototype.pushFileToTarget = function (target, file, source) {
    var self = this;
    var url = '/fwupdate/api/v1/send_file_to_remote';
    self.config.cb.status({
        target: target,
        file: file,
        status: 'push-begin'
    });
    return $.ajax(url, {
        method: 'POST',
        data: {
            source: source,
            file: file,
            target: target
        }
    }).then(function(data, status, xhr) {
        try {
            self.config.cb.status({
                target: target,
                file: file,
                status: 'push-end',
                xhrData: {
                    data: data,
                    status: status,
                    xhr: xhr
                }
            });
        } catch (e) {
            self.logger.error('pushFileToTarget exception', e, arguments);
        } finally {
            return data;
        }
    });
};

/**
 * applyUpdateToTarget
 *  - request target apply previously-pushed update file
 * @param target The target host/ip address
 * @param file The basename file name to apply (must have been
 * pushed previously)
 * @returns {Promise.<T>|*}
 */
FwUpdater.prototype.applyUpdateToTarget = function(target, file) {
    var self = this;
    var url = '/fwupdate/api/v1/apply_update_on_remote';
    self.config.cb.status({
        target: target,
        file: file,
        status: 'apply-request'
    });
    return $.ajax(url, {
        method: 'POST',
        data: {
            target: target,
            file: file,
        }
    }).then(function(data, status, xhr) {
        try {
            self.config.cb.status({
                target: target,
                file: file,
                status: 'apply-end',
                xhrData: {
                    data: data,
                    status: status,
                    xhr: xhr
                }
            });
        } catch (e) {
            self.logger.error('applyUpdateToTarget exception', e, arguments);
        } finally {
            return data;
        }
    });
};

/**
 * uploadAndApplyUpdate
 *
 * @param target The target hostname/ip address
 * @param file The basename file to send from this unit to target.
 * @returns {Promise.<T>|*}
 */
FwUpdater.prototype.uploadAndApplyUpdate = function (target, file, source) {
    var self = this;
    return this.pushFileToTarget(target, file, source).then(function(response) {
        return self.applyUpdateToTarget(target, file);
    });
};

/**
 * updateTargets
 *  - update a list of target IP addresses with the selected file.
 *  TODO: return a promise?
 * @param targets An array of target IP addresses (or hostnames).
 * @param file The base filename to push from this unit to target  and apply
 */
FwUpdater.prototype.updateTargets = function(targets, file, source) {
    var self = this,
        deferred = $.Deferred(),
        itemCount = targets.length,
        endMarker = '@@@'; // invalid hostname, used as end marker.

    targets.push(endMarker);  // marks end of processing

    var res = targets.reduce(function(prev, cur) {
        return prev.then(
            // result is either undefined (as for initial or terminal values)
            // or the xhr/$.ajax result data.
            function (result) {
                // Success, go for next one
                // TODO: remove boilerplate.
                self.logger.info('resolve result, prev, cur', result, prev, cur);
                try {
                    if (result) {
                        var status = {
                            target: result.params.target,
                            file: file,
                            status: 'operation-success',
                            result: result
                        };
                        self.logger.debug('prev is truthy', status);
                        self.config.cb.status(status);
                    }
                } catch (e) {
                    self.logger.error('updateTargets resolve handler exception', e);
                } finally {
                    if (cur === endMarker) {
                        self.logger.debug('cur == end marker, resolve');
                        deferred.resolve('done');
                    } else {
                        return self.uploadAndApplyUpdate(cur, file, source);
                    }
                }
            },
            function (err) {
                // Upload or apply failed
                self.logger.error('reject err, prev, cur', err, prev, cur);
                try {
                    if (result) {
                        var status = ({
                            target: result.params.target,
                            file: file,
                            status: 'operation-error',
                            error: err
                        });
                        self.logger.debug('prev is truthy', status);
                        self.config.cb.status(status);
                    }
                } catch (e) {
                    self.logger.error('updateTargets resolve handler exception', e);
                } finally {
                    if (cur === endMarker) {
                        self.logger.debug('cur == end marker, resolve');
                        deferred.resolve('done');
                    } else {
                        return self.uploadAndApplyUpdate(cur, file, source);
                    }
                }
            }
        );
    }, $.Deferred().resolve());
    return deferred.promise();
}

